@@ -1,4 +1,4 @@
-# Copyright (C)  2012-2017   Mark Seligman
+# Copyright (C)  2012-2018  Mark Seligman
 ##
 ## This file is part of ArboristBridgeR.
 ##
@@ -21,5 +21,5 @@ ForestFloorExport <- function(arbOut) {
 
 
 "ForestFloorExport.Rborist" <- function(arbOut) {
-  return (tryCatch(.Call("RcppForestFloorExport", arbOut), error = function(e) {stop(e)}))
+  return (tryCatch(.Call("ForestFloorExport", arbOut), error = function(e) {stop(e)}))
 }
